# DESCRIPTION - Raptor Fast Submission Library

package Library_raptorfast;

use warnings;
no warnings 'uninitialized';
$|++;
use strict;

use Net::LDAP;
use Net::LDAP::Control::Paged;
use Net::LDAP::Constant qw( LDAP_CONTROL_PAGED );

our($VERSION, $DEBUG);
$VERSION = '2.0';
# Normal State = 0; Debug State = 1
$DEBUG = 0;

#EXCHANGE CONSTANTS
our $ACCOUNTDISABLE = 2;
our $LOCKOUT = 16;

#MASTER LDAP OBJECT HANDLER
sub handle_ldap_object {
  my ($msg, $data) = @_;

  #print "DEBUG: handle_ldap_object - $msg - $data\n";
  if ($data) {
    if (uc(ref($data)) eq 'NET::LDAP::ENTRY') {
      return handle_ldap_entry($msg, $data);
    } elsif (uc(ref($data)) eq 'NET::LDAP::REFERENCE') {
      return handle_ldap_reference($msg, $data);
    } else {
      #UNKNOWN
    }
  }
  return;
}

sub handle_ldap_entry {
  # Net::LDAP::Entry HANDLER
  # This is the key record handler.   
  # Ignore accounts that are disabled or locked out and extract the SMTP email 
  # addresses from the record and output only the SMTP ones we haven't seen before.
  my ($msg, $data) = @_;
  my (@emails, $email, $name, $username, $primary_email);

  #print "DEBUG: ".$data->get_value('proxyAddresses')."\n";
  #print "DEBUG: ".$data->get_value('sAMAccountName')."\n";
  #print "DEBUG: ".$data->get_value('displayName')."\n";
  #print "DEBUG: ".$data->get_value('mail')."\n\n";

  # Extract the SMTP emails from the LDAP data and check each
  @emails = grep { /^smtp:/i } $data->get_value("proxyAddresses");
  $name = $data->get_value('displayName');
  $username = $data->get_value('sAMAccountName');
  $primary_email = $data->get_value('mail');

  # If we found any, strip off the SMTP: identifier and print them out 
  if (int($data->get_value( "userAccountControl" )) & $ACCOUNTDISABLE) {
    #print "DISABLED: ";
  } elsif (int($data->get_value( "userAccountControl" )) & $LOCKOUT) {
    #print "LOCKED OUT: ";
  } else {
    foreach $email (@emails) {
      $email =~ s/^smtp:(.+)$/$1/i;
    }
    push(@main::accounts, {name=>$name, username=>$username, emails=>\@emails, primary_email=>$primary_email});
    print "Found account: $username, primary email: $primary_email, total emails: ". (scalar @emails) ."\n";
  }
}

sub handle_ldap_reference {
  my ($msg, $data) = @_; 
  my ($sub_data);
  
  foreach $sub_data ($data->references()) {
    return handle_ldap_object($msg, $sub_data);
  }
}

sub ldap_search {
  my ($FORM, %params) = @_;
  my ($msg, $entry, $page, @search_args, $resp, $cookie);

  $params{'page_size'} ||= 100;

  if (ref($params{'ldap'}) !~ /^Net::LDAP/i) {
    return (0, 'Invalid LDAP Handle Received');
  }

  # Exchange has a maximum recordset size of 1000 records by default.  Use paging to get the full list.
  $page = Net::LDAP::Control::Paged->new( size => $params{'page_size'} );

  print "Running a Search on a base of: [$params{'base'}] with the following filter: [$params{'filter'}]\n";
  @search_args = ( attrs=>['proxyAddresses','userAccountControl', 'sAMAccountName', 'displayName', 'mail'],callback=>\&Library_raptorfast::handle_ldap_object, control=>[$page],
                   scope=>'subtree', filter=>$params{'filter'}, base=>$params{'base'});

  #DEBUG SEARCH
  if ($params{'debug'}) {
    $msg = $params{'ldap'}->search(scope=>'subtree', filter=>$params{'filter'}, base=>$params{'base'});
    foreach $entry ($msg->all_entries) { 
      print "DEBUG: ";
      print $entry->dump;
      print "\n\n";
    }
    return (1, 'DEBUG: Search Results Returned');
  } else {
    #SEARCH AND PARSE THE RESULTS NORMALLY
    while(1) {
      # Perform search
      $msg = $params{'ldap'}->search(@search_args);

      # Only continue on LDAP_SUCCESS
      if ($msg->code) { last };

      # Get cookie from paged control
      ($resp) = $msg->control( LDAP_CONTROL_PAGED ) or last;

      $cookie = $resp->cookie or last;

      # Set cookie in paged control
      $page->cookie($cookie);
    }
    if ($cookie) {
      # We had an abnormal exit, so let the server know we do not want any more
      $page->cookie($cookie);
      $page->size(0);
      $params{'ldap'}->search( @search_args );
    }
    return (1, 'Search Results Returned');
  }
}

sub print_ldap_error {
  print &get_ldap_error(@_);
}

sub get_ldap_error {
  my ($FORM, %params) = @_;
  my ($output);

  if (ref($params{'msg'}) =~ /^Net::LDAP/i) {
    if ($params{'msg'}->code()) {
       $output = "Error: LDAP Code:".$params{'msg'}->code()." / Error: ".$params{'msg'}->error." / Error Text:".$params{'msg'}->error_text()."\n";
    }
  } else {
    $output = "Error: Invalid LDAP Message Handle\n";
  }

  return $output;
}

sub ldap_unbind_and_disconnect {
  my ($FORM, %params) = @_;

  if (uc(ref($params{'ldap'})) eq 'NET::LDAP') {
    $params{'ldap'}->unbind();
    $params{'ldap'}->disconnect();
    undef $params{'ldap'};
    print "DEBUG: Disconnected.\n" if ($params{'debug'});
    return (0, 'Unbound and Disconnected from LDAP server.');
  } else {
    return (1, 'Invalid LDAP handle passed for Unbind / Disconnect.');
  }
}

sub ldap_connect_and_bind {
  my ($FORM, %params) = @_;
  my ($i, $ldap, $msg);

  #Connect to the server - try each one by one until we succeed
  $i = 0;
  while ($i < scalar(@{$params{'servers'}}) && !$ldap) {
    print "Trying to Connect to LDAP Server #$i: $params{'servers'}->[$i]\n";
    $ldap = Net::LDAP->new( $params{'servers'}->[$i] );
  }

  #CONNECT STATUS
  if (uc(ref($ldap)) eq 'NET::LDAP') {
    print "Connected!\n";

    print "DEBUG: Bind is: $params{'bind'} / $params{'password'}\n" if ($params{'debug'});

    #BIND (LOGIN) TO THE LDAP SERVER
    $msg = $ldap->bind( dn=>$params{'bind'}, password=>$params{'password'} );

    # If we can't bind, we can't continue
    if( $msg->code() ) {
      return (0, &Library_raptorfast::get_ldap_error($FORM, msg=>$msg));
    } else {
      print "DEBUG: Login Successful!\n" if ($params{'debug'});
      return (1, "Login Successful!\n", $ldap);
    }

  } else {
    return (0, 'Unable to connect to any LDAP servers!');
  }
}

sub test_perl {
  #TEST FOR PERL FUNCTIONALITY
  my ($i);
  for ($i = 0; $i < 1000000; $i++) {
    print "Hello World!";
  }
}

1;
